@@ -124,37 +124,37 @@ impl<P: Printer> Worker for ScanWithConfig<P> {
   type Item = (PathBuf, AstGrep, BitSet);
   fn consume_items(&self, items: Items<Self::Item>) -> Result<()> {
     self.printer.before_print()?;
-    let mut has_error = 0;
+    let mut error_count = 0usize;
     for (path, grep, hit_set) in items {
       let file_content = grep.source().to_string();
       let path = &path;
       let rules = self.configs.for_path(path);
       let combined = CombinedScan::new(rules);
       let interactive = self.arg.output.needs_interactive();
+      // exclude_fix rule because we already have diff inspection before
+      let scanned = combined.scan(&grep, hit_set, /* separate_fix*/ interactive);
       if interactive {
-        let diffs = combined
-          .diffs(&grep, hit_set.clone())
+        let diffs = scanned
+          .diffs
           .into_iter()
-          .map(|(nm, idx)| {
+          .map(|(idx, nm)| {
             let rule = combined.get_rule(idx);
             (nm, rule)
           })
           .collect();
         match_rule_diff_on_file(path, diffs, &self.printer)?;
       }
-      // exclude_fix rule because we already have diff inspection before
-      let matched = combined.scan(&grep, hit_set, /* exclude_fix*/ interactive);
-      for (idx, matches) in matched {
+      for (idx, matches) in scanned.matches {
         let rule = combined.get_rule(idx);
         if matches!(rule.severity, Severity::Error) {
-          has_error += 1;
+          error_count = error_count.saturating_add(matches.len());
         }
         match_rule_on_file(path, matches, rule, &file_content, &self.printer)?;
       }
     }
     self.printer.after_print()?;
-    if has_error > 0 {
-      Err(anyhow::anyhow!(EC::DiagnosticError(has_error)))
+    if error_count > 0 {
+      Err(anyhow::anyhow!(EC::DiagnosticError(error_count)))
     } else {
       Ok(())
     }
@@ -203,23 +203,23 @@ impl<P: Printer> Worker for ScanWithRule<P> {
   type Item = (PathBuf, AstGrep, BitSet);
   fn consume_items(&self, items: Items<Self::Item>) -> Result<()> {
     self.printer.before_print()?;
-    let mut has_error = 0;
+    let mut error_count = 0usize;
     let combined = CombinedScan::new(self.rules.iter().collect());
     for (path, grep, hit_set) in items {
       let file_content = grep.source().to_string();
       // do not exclude_fix rule in run_with_rule
-      let matched = combined.scan(&grep, hit_set, false);
-      for (idx, matches) in matched {
+      let scanned = combined.scan(&grep, hit_set, false);
+      for (idx, matches) in scanned.matches {
         let rule = combined.get_rule(idx);
         if matches!(rule.severity, Severity::Error) {
-          has_error += 1;
+          error_count = error_count.saturating_add(matches.len());
         }
         match_rule_on_file(&path, matches, rule, &file_content, &self.printer)?;
       }
     }
     self.printer.after_print()?;
-    if has_error > 0 {
-      Err(anyhow::anyhow!(EC::DiagnosticError(has_error)))
+    if error_count > 0 {
+      Err(anyhow::anyhow!(EC::DiagnosticError(error_count)))
     } else {
       Ok(())
     }
