@@ -73,8 +73,29 @@ impl<L: Language> SerializableRuleConfig<L> {
   }
 
   pub fn get_matcher(&self, globals: &GlobalRules<L>) -> Result<RuleCore<L>, RuleConfigError> {
-    let env = DeserializeEnv::new(self.language.clone()).with_globals(globals);
-    self.core.get_matcher(env)
+    // every RuleConfig has one rewriters, and the rewriter is shared between sub-rules
+    // all RuleConfigs has one common globals
+    // every sub-rule has one util
+    let rewriters = GlobalRules::default();
+    let env = DeserializeEnv::new(self.language.clone())
+      .with_globals(globals)
+      .with_rewriters(&rewriters);
+    let rule = self.core.get_matcher(env)?;
+    let Some(ser) = &self.rewriters else {
+      return Ok(rule);
+    };
+    for val in ser {
+      // NB should inherit env from matcher to inherit utils
+      // TODO: optimize duplicate env creation/util registraion
+      let env = DeserializeEnv::new(self.language.clone())
+        .with_globals(globals)
+        .with_rewriters(&rewriters);
+      let env = self.get_deserialize_env(env)?;
+      let env = val.core.get_deserialize_env(env)?;
+      let rewriter = val.core.get_matcher_from_env(&env)?;
+      rewriters.insert(&val.id, rewriter).expect("should work");
+    }
+    Ok(rule)
   }
 }
 
@@ -101,21 +122,7 @@ impl<L: Language> RuleConfig<L> {
     inner: SerializableRuleConfig<L>,
     globals: &GlobalRules<L>,
   ) -> Result<Self, RuleConfigError> {
-    let mut matcher = inner.get_matcher(globals)?;
-    let Some(ser) = inner.rewriters.clone() else {
-      return Ok(Self { inner, matcher });
-    };
-    let mut rewriters = HashMap::new();
-    for val in ser {
-      // NB should inherit env from matcher to inherit utils
-      // TODO: optimize duplicate env creation/util registraion
-      let env = DeserializeEnv::new(inner.language.clone()).with_globals(globals);
-      let env = inner.get_deserialize_env(env)?;
-      let env = val.core.get_deserialize_env(env)?;
-      let rewriter = val.core.get_matcher_from_env(&env)?;
-      rewriters.insert(val.id, rewriter);
-    }
-    matcher.add_rewriters(rewriters)?;
+    let matcher = inner.get_matcher(globals)?;
     Ok(Self { inner, matcher })
   }
 
@@ -443,4 +450,32 @@ rewriters:
     let b = nm.get_env().get_transformed("B").expect("should have");
     assert_eq!(String::from_utf8_lossy(b), "yjsnp");
   }
+
+  #[test]
+  fn test_use_rewriter_recursive() {
+    let rule: SerializableRuleConfig<TypeScript> = from_str(
+      r"
+id: test
+rule: {pattern: 'a = $A'}
+language: Tsx
+transform:
+  B: { rewrite: { rewriters: [re], source: $A } }
+rewriters:
+- id: handle-num
+  rule: {regex: '114'}
+  fix: '1919810'
+- id: re
+  rule: {kind: number, pattern: $A}
+  transform:
+    B: { rewrite: { rewriters: [handle-num], source: $A } }
+  fix: $B
+    ",
+    )
+    .expect("should parse");
+    let rule = RuleConfig::try_from(rule, &Default::default()).expect("work");
+    let grep = TypeScript::Tsx.ast_grep("a = 114514");
+    let nm = grep.root().find(&rule.matcher).unwrap();
+    let b = nm.get_env().get_transformed("B").expect("should have");
+    assert_eq!(String::from_utf8_lossy(b), "1919810");
+  }
 }
