@@ -45,18 +45,25 @@ impl Rewrite {
     let rules: Vec<_> = self
       .rewriters
       .iter()
-      .filter_map(|id| rewriters.get(id))
+      .filter_map(|id| rewriters.get(id)) // TODO: better handling
       .collect();
     let edits = find_and_make_edits(nodes, &rules);
     let rewritten = if let Some(joiner) = &self.join_by {
       let mut ret = vec![];
       let mut edits = edits.into_iter();
       if let Some(first) = edits.next() {
+        let mut pos = first.position - start + first.deleted_length;
         ret.extend(first.inserted_text);
         let joiner = D::Source::decode_str(joiner);
         for edit in edits {
+          let p = edit.position - start;
+          // skip overlapping edits
+          if pos > p {
+            continue;
+          }
           ret.extend_from_slice(&joiner);
           ret.extend(edit.inserted_text);
+          pos = p + edit.deleted_length;
         }
         ret
       } else {
